@@ -15,24 +15,14 @@ from _fui import _Fui
 
 def run():
     sys_argv = sys.argv[1:]
-
     f = _Fui()
-
-    optlist , args  = getopt.getopt( sys_argv, '' )
-    
+    optlist , args  = getopt.getopt( sys_argv, '' )    
     if len(args) < 1: f.start()
-
     for a in args:
         # arg 'ts' with test
-        if a in ['s', 'ts' ,'st', 'start' ]:
-            f.start()
-        if a in [ 'm' , 'msg' , 'msgfmt' ]:
-            f.msgfmt()
-        
-        if a in ['pip' , 'pip_install']:
-            f.pip_install_r()
-        
-        if a in [ 'init', 'initial' ]:
-            f.initialize()
+        if a in [ 's', 'ts' ,'st', 'start' ]:   f.start()
+        if a in [ 'm' , 'msg' , 'msgfmt' ]:     f.msgfmt()
+        if a in [ 'pip' , 'pip_install']:       f.pip_install_r()
+        if a in [ 'init', 'initial' ]:          f.initialize()
 
         
\ No newline at end of file
