@@ -24,6 +24,7 @@ import pickle
 import yaml
 import uuid
 import time
+import re
 
 
 class _MainUI():
@@ -48,6 +49,12 @@ class _MainUI():
         self.known_encodings = dict(\
             pickle.load( open(face_encodings_path , 'rb') ))
         self.comparison_tolerance = comparison_tolerance
+        self.image_exts = ['jpg','png']
+        self.video_exts = ['mp4','avi','3gp','webm','mkv']
+        self.showf_sv = None;       self.rec_img = False
+        self.showfm = self.mainui.showframe
+        self.entryfm = self.mainui.entryframe
+        self.addinfofm = self.mainui.addinfoframe
             
         try:self.screenwidth = self.mainui.root.winfo_screenwidth();\
             self.screenheight = self.mainui.root.winfo_screenheight()
@@ -59,45 +66,60 @@ class _MainUI():
         self.mainui.mainloop()
     
     def set_ui_events( self ):
-
         self.mainui.langcombobox.lang_combobox.bind('<<ComboboxSelected>>',
             self.change_language )
-        self.mainui.showframe.ct_entry.bind('<FocusOut>', self.save_ct )
-        self.mainui.showframe.rec_button['command'] = self.rec_now
-        self.mainui.entryframe.prev_f_button['command'] = \
+        self.showfm.ct_entry.bind('<FocusOut>', self.save_ct )
+        self.showfm.rec_button['command'] = self.rec_now
+        self.showfm.showf_go_btn['command'] = self.showf_go
+        self.showfm.showf_optionmenu_sv.trace('w', self.show_from )
+        self.entryfm.prev_f_button['command'] = \
             lambda: self.pick_face_by_num(-1)
-        self.mainui.entryframe.next_f_button['command'] = \
+        self.entryfm.next_f_button['command'] = \
             lambda: self.pick_face_by_num(1)
-        self.mainui.entryframe.save_button['command'] = self.save_db_encoding
-        self.mainui.showframe.show_f_optionmenu_var.trace('w', self.show_from )
+        self.entryfm.save_button['command'] = self.save_db_encoding
         self.mainui.root.protocol("WM_DELETE_WINDOW", self.destroy )
-        self.mainui.addinfoframe.add_rf_button['command'] = self.ins_rf 
+        self.addinfofm.add_rf_button['command'] = self.ins_rf 
          
     def destroy( self ):
-        if self.iru is not None:
-            self.iru.vid_release()
+        if self.iru is not None: self.iru.vid_release()
         exit()
 
 # SHOW_FRAME FUNCTIONS 
 ###############################################################################
 
     def save_ct( self , event):
-        ct_stringvar_get = float(self.mainui.showframe.ct_stringvar.get())
+        ct_stringvar_get = float(self.showfm.ct_stringvar.get())
         self.comparison_tolerance = comparison_tolerance = ct_stringvar_get
         setting_yml['comparison_tolerance'] = ct_stringvar_get
         yaml.dump( setting_yml, open( setting_path, 'w') )
+    
+    def showf_go( self):
+        self.showf_sv = self.showfm.showf_sv.get()
+        self.rec_img = False
+        self.after_cancel()
 
+        showf_ext = self.showf_sv.split('.')[-1]
+
+        if showf_ext in self.video_exts:
+            self.video_source = self.showf_sv;  self.play_video()
+            return
+        if re.match(r'\d+', self.showf_sv):
+            self.video_source = int(self.showf_sv);  self.play_video()
+            return
+        if showf_ext in self.image_exts:
+            self.pause = True;  self.rec_img = True;    self.view_image()
+            return 
+        self.showfm.showf_sv.set('')
+        self.show_nsrc_error()
+            
     def show_from( self, *args  ):
-        keys =  self.mainui.showframe.show_from_optionmenus.keys()
-        values = self.mainui.showframe.show_from_optionmenus.values()
-        value = self.mainui.showframe.show_f_optionmenu_var.get()
+        keys =  self.showfm.showf_t_dict.keys()
+        values = self.showfm.showf_t_dict.values()
+        value = self.showfm.showf_optionmenu_sv.get()
         show_f = list(keys)[ list( values ).index( value )]
         
-        image_exts = ['jpg','png']
-        video_exts = ['mp4','avi','3gp','webm','mkv']
-
-        if self.root_after != -1:
-            self.mainui.root.after_cancel( self.root_after )
+        self.after_cancel()
+        self.rec_img = False
     
         if show_f == 'file':
             if self.iru is not None: self.iru.vid_release() ; self.iru = None
@@ -106,31 +128,41 @@ class _MainUI():
                 title = _('Select a file'),
                 filetypes = [\
                     ( _('Image or video'), \
-                    '*.'+(' *.'.join( image_exts + video_exts)))\
+                    '*.'+(' *.'.join( self.image_exts + self.video_exts)))\
                 ],
                 initialdir = '~'
             )
 
             if len( self.face_src_path ) > 0:
                 ext = os.path.splitext( self.face_src_path )[1][1:]
-                if ext in image_exts:
-                    self.mainui.showframe.rec_button['state'] = 'disabled'
+
+                self.showfm.showf_sv.set( self.face_src_path )
+
+                if ext in self.image_exts:
                     # Pause
                     self.pause = True
+                    self.rec_img = True
                     self.view_image()
-                else: self.mainui.showframe.rec_button['state'] = 'normal'
-
-                if ext in video_exts:
+                if ext in self.video_exts:
                     self.video_source = self.face_src_path
                     self.play_video()
 
-        elif show_f == 'camara':
+        elif show_f == 'camera':
             self.video_source = 0
+            self.showfm.showf_sv.set( self.video_source )
             self.play_video()
-
+    
+    def after_cancel( self ):
+        if self.root_after != -1:
+            self.mainui.root.after_cancel( self.root_after )
+        
     def play_video( self ):
         if self.iru is not None: self.iru.vid_release()
         self.iru = IRU( self.video_source )
+        if not self.iru.vid.isOpened():
+            self.show_nsrc_error()
+            if debug: print('play_video: Unable to open video source')
+            return
         # Play
         self.pause = False
         self.get_resize_fxfy()
@@ -140,46 +172,42 @@ class _MainUI():
             print('fps: ', self.iru.fps)
            
 
-    def rec_now( self ):
-        
+    def rec_now( self ):        
+        if self.rec_img: return
+        if self.face_sum < 1:
+            if debug: print('No face detected')
+            return
         if self.iru is None: self.show_nfd_info() ; return
-
         if self.pause:
             # Play
             self.pause = False
-            self.mainui.showframe.rec_stringvar.set(_('Recognize'))
+            self.showfm.rec_stringvar.set(_('Recognize'))
             self.refresh_frame()
             self.update_ui()
             self.curr_face_id = None
         else:
             # Pause
             self.pause = True
-            self.mainui.showframe.rec_stringvar.set( _('Play') )
+            self.showfm.rec_stringvar.set( _('Play') )
 
 
     def view_image( self ):
-
         face_image  = cv2.imread( self.face_src_path )
         self.iru_frame = cv2.cvtColor( face_image, cv2.COLOR_BGR2RGB)
 
         img = Image.fromarray( self.iru_frame )
         imgtk = ImageTk.PhotoImage( image= img )
-        self.mainui.showframe.vid_frame_label.imgtk = imgtk
-        self.mainui.showframe.vid_frame_label.configure(image=imgtk)
+        self.showfm.vid_frame_label.imgtk = imgtk
+        self.showfm.vid_frame_label.configure(image=imgtk)
 
         self.get_f_loc_and_sum()
         
-        if self.face_sum > 0:
-            self.correct_f_num_ui()
-            self.pick_face_by_num()    
+        if self.face_sum < 1 : return
+        self.correct_f_num_ui()
+        self.pick_face_by_num()    
 
     def refresh_frame( self ):
         if self.iru is None: return
-        if self.pause: 
-            self.update_ui()
-            self.pick_face_by_num()
-            return
-        
         self.vid_ret_frame  = self.iru.get_ret_frame()
         self.iru_frame = self.vid_ret_frame[1]
 
@@ -187,18 +215,20 @@ class _MainUI():
             if debug: print('No frame rect returned.'); return
         
         self.get_f_loc_and_sum()
-
         if self.face_sum > 0:
-
-            self.correct_f_num_ui()
-            self.pick_f_mk_rect()
+            if self.pause:
+                self.update_ui()
+                self.pick_face_by_num()
+                return
+            else:
+                self.correct_f_num_ui()
+                self.mk_frame_rect()
 
         self.update_vid_frame()
         self.root_after = self.mainui.root.after( \
             int(1000/self.iru.fps) , self.refresh_frame )
     
     def update_vid_frame( self ):
-
         if self.fxfy is None: 
             if debug: print('self.fxfy is None'); 
             return
@@ -210,9 +240,8 @@ class _MainUI():
             fx = self.fxfy, fy = self.fxfy )
         vid_img = Image.fromarray( vid_img )
         imgtk = ImageTk.PhotoImage( image=vid_img )
-        self.mainui.showframe.vid_frame_label.imgtk = imgtk
-        self.mainui.showframe.vid_frame_label.configure(image=imgtk)
-
+        self.showfm.vid_frame_label.imgtk = imgtk
+        self.showfm.vid_frame_label.configure(image=imgtk)
 
     def get_resize_fxfy( self ):
         if self.iru.width == self.iru.height == 0: 
@@ -227,7 +256,6 @@ class _MainUI():
         if debug:
             print('self.fxfy: ', self.fxfy)
     
-
     def get_f_loc_and_sum( self ):
         self.get_f_loc()
         self.get_f_sum()
@@ -253,14 +281,12 @@ class _MainUI():
         if self.face_sum > 0 :
             self.curr_f_encoding = face_recognition.face_encodings( \
                 self.iru_frame, [self.face_locations[ self.face_num ]] )[0]
-
      
     def get_curr_f_encoding_and_id( self ):
         self.get_curr_f_encoding()
         self.get_curr_f_id()
                 
     def mk_frame_rect( self ):
-
         for top, right, bottom, left in self.face_locations:
             cv2.rectangle( self.iru_frame, (left, top), \
             (right, bottom), (0, 0, 255), 2)
@@ -273,6 +299,11 @@ class _MainUI():
         messagebox.showerror( _('Error'), \
             _('Check the DOB entry please!') ) 
 
+    def show_nsrc_error( self ):
+        unable_open_s = _('Unable to open video source')
+        messagebox.showerror(  unable_open_s, unable_open_s+': '+self.showf_sv )
+
+
 ###############################################################################
 # SHOW_FRAME FUNCTIONS END
 
@@ -285,13 +316,13 @@ class _MainUI():
         self.face_num = 0 if self.face_num < 0 else \
              self.face_num if self.face_num < self.face_sum \
             else self.face_sum-1
-        self.mainui.entryframe.face_num_stringvar.set( \
+        self.entryfm.face_num_stringvar.set( \
             f'{self.face_num+1}/{self.face_sum}' )
         
     @db_session
     def update_ui(self):
         '''
-        Update self.mainui.entryframe and self.mainui.addinfoframe
+        Update self.entryfm and self.addinfofm
         '''
 
         if self.iru_frame is None: self.show_nfd_info() ; return
@@ -302,20 +333,18 @@ class _MainUI():
         if self.pause:
             if self.curr_face_id is None: return
 
-            p = select(p for p in Person \
-                if p.id == self.curr_face_id ).first()
-            if p is None: return
+            p = Person.get(id = self.curr_face_id)
 
-            self.mainui.entryframe.clear_content()
-            
-            self.mainui.entryframe.uuid_entry['state'] = 'normal'
-            self.mainui.entryframe.uuid_entry.insert(0 , p.id)
-            self.mainui.entryframe.uuid_entry['state'] = 'disabled'
-            self.mainui.entryframe.name_entry.insert(0 , p.name)
-            self.mainui.entryframe.gender_entry.insert(0 , p.gender)
-            self.mainui.entryframe.DOB_entry.insert(0 , p.dob )
-            self.mainui.entryframe.address_entry.insert(0 , p.address)
-            self.mainui.entryframe.note_text.insert(END , p.note)
+            if p is None: return
+            self.entryfm.clear_content()            
+            self.entryfm.uuid_entry['state'] = 'normal'
+            self.entryfm.uuid_entry.insert(0 , p.id)
+            self.entryfm.uuid_entry['state'] = 'disabled'
+            self.entryfm.name_entry.insert(0 , p.name)
+            self.entryfm.gender_entry.insert(0 , p.gender)
+            self.entryfm.DOB_entry.insert(0 , p.dob )
+            self.entryfm.address_entry.insert(0 , p.address)
+            self.entryfm.cmt_text.insert(END , p.comment)
     
             # update addinfoframe
             i_s = select( i for i in PersonInfo \
@@ -323,46 +352,43 @@ class _MainUI():
                 
             for i in i_s:
                 self.ins_rf( il_entry_value=i.label, v_value = i.value,\
-                    note_value = i.note,    frame_name  = i.id )
+                    cmt_value = i.comment, frame_name  = i.id )
 
         else:
-                self.mainui.entryframe.uuid_entry['state'] = 'normal'
-                self.mainui.entryframe.uuid_entry.delete(0, END)
-                self.mainui.entryframe.uuid_entry['state'] = 'disabled'
-                self.mainui.entryframe.name_entry.delete(0, END)
-                self.mainui.entryframe.gender_entry.delete(0 , END)
-                self.mainui.entryframe.DOB_entry.delete(0, END)
-                self.mainui.entryframe.address_entry.delete(0, END)
-                self.mainui.entryframe.note_text.delete('1.0', END)
+            self.entryfm.uuid_entry['state'] = 'normal'
+            self.entryfm.uuid_entry.delete(0, END)
+            self.entryfm.uuid_entry['state'] = 'disabled'
+            self.entryfm.name_entry.delete(0, END)
+            self.entryfm.gender_entry.delete(0 , END)
+            self.entryfm.DOB_entry.delete(0, END)
+            self.entryfm.address_entry.delete(0, END)
+            self.entryfm.cmt_text.delete('1.0', END)
 
-                self.rm_all_ins_rfs()
+            self.rm_all_ins_rfs()
     
     def pick_f_mk_rect( self ):
         '''
-        Pick face image to self.entryframe.face_label and make rectangles for 
-        self.showframe.vid_frame_label.
+        Pick face image to self.entryfm.face_label and make rectangles for 
+        self.showfm.vid_frame_label.
         '''
         self.pick_face_by_num()
         self.mk_frame_rect()
     
     def pick_face_by_num( self, p_n = 0 ):
         '''
-        Pick face image to self.mainui.entryframe.face_label
+        Pick face image to self.entryfm.face_label
         '''
-
-        if not self.pause: return
-        self.face_num = self.face_num + p_n
-
-        self.correct_f_num_ui()
-                   
-        if self.face_num < 0:
-            self.show_nfd_info()
-            if debug:
-                print('No face detected')
+        if not self.pause: 
+            print('Video is playing')
             return
-
+        self.get_f_sum()
+        if self.face_sum < 1: 
+            if debug: print('No face detected')
+            return
+        self.face_num = self.face_num + p_n
+        self.correct_f_num_ui()
+        
         top, right, bottom, left = self.face_locations[ self.face_num ]
-
         b_t_sub = bottom - top
         r_l_sub = right - left
         size_add = b_t_sub if  b_t_sub > r_l_sub else r_l_sub
@@ -372,9 +398,8 @@ class _MainUI():
         frame = cv2.resize( frame, (200, 200) )
         face_img = Image.fromarray( frame )
         faceimgtk = ImageTk.PhotoImage( image=face_img )
-        self.mainui.entryframe.face_label.imgtk = faceimgtk
-        self.mainui.entryframe.face_label.configure(image=faceimgtk)
-
+        self.entryfm.face_label.imgtk = faceimgtk
+        self.entryfm.face_label.configure(image=faceimgtk)
 
     @db_session
     def save_db_encoding( self ):
@@ -391,42 +416,42 @@ class _MainUI():
                 print( 'Person exists')
             p = select(p for p in Person if \
                 p.id == self.curr_face_id ).first()
-            p.dob = self.mainui.entryframe.DOB_entry.get()
-            p.name = self.mainui.entryframe.name_entry.get()
-            p.gender = self.mainui.entryframe.gender_entry.get()
-            p.note = self.mainui.entryframe.note_text.get(1.0, 'end')
+            p.dob = self.entryfm.DOB_entry.get()
+            p.name = self.entryfm.name_entry.get()
+            p.gender = self.entryfm.gender_entry.get()
+            p.comment = self.entryfm.cmt_text.get(1.0, 'end')
 
         else:
             if debug:
                 print('New person')
             p = Person( id = str(uuid.uuid4()),\
-                name = self.mainui.entryframe.name_entry.get() ,
-                gender = self.mainui.entryframe.gender_entry.get(),
-                dob = self.mainui.entryframe.DOB_entry.get(), 
-                address = self.mainui.entryframe.address_entry.get(),
-                note = self.mainui.entryframe.note_text.get(1.0, 'end') )
+                name = self.entryfm.name_entry.get() ,
+                gender = self.entryfm.gender_entry.get(),
+                dob = self.entryfm.DOB_entry.get(), 
+                address = self.entryfm.address_entry.get(),
+                comment = self.entryfm.cmt_text.get(1.0, 'end') )
         
         if debug:
-            print( self.mainui.addinfoframe.ins_vars )
+            print( self.addinfofm.ins_vars )
 
-        for frame_name, info_widgets in self.ins_vars.items():
-            label_value = info_widgets[0].get()
-            value_v = info_widgets[1].get()
-            note_value = info_widgets[2].get()
+        for frame_name, sv in self.ins_vars.items():
+            label_value= sv[0].get()
+            value_v = sv[1].get()
+            cmt_value = sv[2].get()
 
             if debug:
-                print( label_value,  value_v, note_value)
+                print( label_value,  value_v, cmt_value)
 
-            if len(label_value+  value_v+ note_value )< 1: continue
+            if len(label_value+  value_v+ cmt_value )< 1: continue
             
             p_i = PersonInfo.get(person_id =p.id, label =label_value)
             if p_i:
                 if debug: print( 'PersonInfo exists' )
-                p_i.value = value_v;    p_i.note = note_value
+                p_i.value = value_v;    p_i.comment = cmt_value
             else:
                 if debug: print( 'New PersonInfo' )
                 p_i = PersonInfo( id = frame_name,
-                    note = note_value,      person_id = p.id,      
+                    comment = cmt_value,      person_id = p.id,      
                     label = label_value,    value = value_v
                 )
 
@@ -450,35 +475,39 @@ class _MainUI():
 ###############################################################################
 
     def ins_rf( self, frame_name = '', il_entry_value='',  v_value = '', \
-        note_value = ''  ):
+        cmt_value = ''  ):
 
         frame_name = str( uuid.uuid4() ) if len( frame_name )<1 else frame_name
-        row_frame = tk.Frame(self.mainui.addinfoframe.frame, name =frame_name )
+        row_frame = tk.Frame(self.addinfofm.frame, name = frame_name )
 
         info_frame = tk.Frame( row_frame )
         del_button = tk.Button( row_frame , text = _('Delete'), \
             command =lambda: self.rm_ins_rf(frame_name) )
         
-        tk.Label( info_frame, text = _('Label') ).grid( column =  0, row =  0 )
-        il_entry_svar = StringVar( info_frame , value = il_entry_value)
-        tk.Entry(info_frame, textvariable= il_entry_svar).grid(column=1, row= 0)
+        il_entry_sv = StringVar( info_frame , value = il_entry_value )
+        tk.Entry(info_frame, textvariable= il_entry_sv, justify = RIGHT)\
+            .grid( column=1, row= 0)
 
-        tk.Label( info_frame, text=_('Value')).grid( column = 0, row = 2)
-        value_sv = StringVar( info_frame, value = v_value )
-        tk.Entry( info_frame, textvariable = value_sv).grid(column = 1,row= 2 )
+        tk.Label(info_frame, text = ':').grid(column = 2, row = 0)
 
-        tk.Label( info_frame, text=_('Note')).grid( column = 0, row = 3)
-        note_sv = StringVar( info_frame, note_value )
-        tk.Entry( info_frame, textvariable = note_sv ).grid(column =1 ,row= 3 )
+        value_sv = StringVar( info_frame, value = v_value)
+        tk.Entry( info_frame, textvariable = value_sv).grid(\
+            column = 3,row= 0)
+
+        tk.Label(info_frame, text = f"{_('Comment')}:")\
+            .grid(column = 4, row = 0)
+        cmt_sv = StringVar( info_frame, cmt_value)
+        tk.Entry( info_frame, textvariable = cmt_sv ).grid(column = 5 ,row= 0 )
         
         info_frame.grid(column = 0, row = 0)
         del_button.grid(column = 1, row = 0)
+
         row_frame.pack( side = TOP )
 
         ttk.Separator(info_frame, orient='horizontal')\
             .place(relx=0, rely=0, relwidth=1, relheight=0.01)
         
-        self.ins_vars[frame_name] = [il_entry_svar, value_sv, note_sv]
+        self.ins_vars[frame_name] = [il_entry_sv , value_sv, cmt_sv ]
         
         if debug:
             print( self.ins_vars )
@@ -486,7 +515,7 @@ class _MainUI():
     @db_session
     def rm_ins_rf( self , frame_name):
         # update UI
-        self.mainui.addinfoframe.frame.nametowidget(frame_name).pack_forget()
+        self.addinfofm.frame.nametowidget(frame_name).pack_forget()
         self.ins_vars.pop(frame_name)
         if debug:
             print( frame_name, self.ins_vars )
@@ -500,7 +529,7 @@ class _MainUI():
     
     def rm_all_ins_rfs(self):
         for i in self.ins_vars.keys():
-            self.mainui.addinfoframe.frame.nametowidget(i).pack_forget()
+            self.addinfofm.frame.nametowidget(i).pack_forget()
         self.ins_vars = {}
              
 ###############################################################################
@@ -544,6 +573,7 @@ class _MainUI():
                 self.comparison_tolerance)
                         
             if True in comparisons: self.curr_face_id = _id ; break
+
         
 ###############################################################################
 # OTHER FUNCTIONS END
@@ -580,7 +610,7 @@ class IRU():
     def show_vid_src_error( self ):
         messagebox.showerror( 
             _('Unable to open video source'), self.video_source )
-        
+
     def vid_release( self ):
         if self.vid is None: return
         self.vid.release()
