@@ -15,16 +15,12 @@ class MainUI():
     def __init__(self):
         self.root = tk.Tk()
         self.root.title( _('Funing')+' ' + version )
-
         # frame
         self.showframe = ShowFrame( tk.Frame( self.root ) ) 
-
         # entry_frame
         self.entryframe = EntryFrame( tk.Frame( self.root ) )
-
         # lang_combobox
-        self.langcombobox = LangCombobox( self.root )
-        
+        self.langcombobox = LangCombobox( self.root )        
         # addinfoframe
         self.addinfoframe = AddInfoFrame( tk.Frame( self.root ) )
 
@@ -75,24 +71,21 @@ class ShowFrame():
         # video label
         self.vid_frame_label = tk.Label( self.frame )
 
-        self.show_from_optionmenus =  {
-            'file':_('File'),
-            'camara': _('Camara') }
-
-        self.show_f_optionmenu_var = tk.StringVar( self.frame )
-        self.show_f_optionmenu_var.set( _('Open') )
-        
-        self.show_from_optionmenu = tk.OptionMenu( 
-            self.frame, self.show_f_optionmenu_var , 
-            *self.show_from_optionmenus.values() )
+        self.showf_sv = tk.StringVar( self.frame )
+        self.showf_entry = tk.Entry( \
+            self.frame , width = 10, textvariable = self.showf_sv)
+        self.showf_go_btn = tk.Button(self.frame, text = _('GO') )
 
+        self.showf_t_dict =  { 'file':_('File'), 'camera': _('Camera') }
+        self.showf_optionmenu_sv = tk.StringVar(self.frame, value = _('Open'))
+        self.showf_optionmenu = tk.OptionMenu( self.frame, \
+            self.showf_optionmenu_sv , *self.showf_t_dict.values() )
 
         # comparison_tolerance entry
         self.ct_label = tk.Label( \
-            self.frame, text = _('tolerance:') )
+            self.frame, text = _('tolerance') + ':' )
         self.ct_stringvar = StringVar( frame, ct )
-        self.ct_entry = tk.Entry( \
-            self.frame, width = 8,\
+        self.ct_entry = tk.Entry( self.frame, width = 8,\
             textvariable = self.ct_stringvar )
 
         # shoot
@@ -104,14 +97,16 @@ class ShowFrame():
 
         # place vid_frame_label
         self.vid_frame_label.grid( column = 0, row = 0, rowspan = 3,
-            columnspan = 4 )
+            columnspan = 6 )
 
+        self.showf_entry.grid( column = 0, row = 4, sticky = E)
+        self.showf_go_btn.grid( column = 1, row = 4, sticky = W)
+        self.showf_optionmenu.grid( column = 2, row = 4 , sticky = W)
+        self.ct_label.grid( column = 3, row = 4, sticky = E )
+        self.ct_entry.grid( column = 4, row = 4, sticky = W )
+        self.rec_button.grid( column = 5, row = 4)
+        
         # place frame
-        self.show_from_optionmenu.grid( column = 0, row = 4 )
-        self.ct_label.grid( column = 1, row = 4, sticky = E )
-        self.ct_entry.grid( column = 2, row = 4, sticky = W )
-        self.rec_button.grid( column = 3, row = 4)
-
         self.frame.grid( column = 0, row = 0 )
 
              
@@ -158,8 +153,8 @@ class EntryFrame():
         self.address_label = tk.Label( self.frame, text = _('Address') )
         self.address_entry = tk.Entry( self.frame )
 
-        self.note_label = tk.Label( self.frame, text = _('Note') )
-        self.note_text = tk.Text( self.frame, height = 6, width = 20 )
+        self.cmt_label = tk.Label( self.frame, text = _('Comment') )
+        self.cmt_text = tk.Text( self.frame, height = 6, width = 20 )
 
         # save_button
         self.save_button = tk.Button( self.frame, text = _('Save') )
@@ -172,7 +167,7 @@ class EntryFrame():
         self.name_entry.delete(0, END)
         self.DOB_entry.delete(0, END)
         self.address_entry.delete(0, END)
-        self.note_text.delete('1.0', END)
+        self.cmt_text.delete('1.0', END)
         
     def place( self ):
         # place frame
@@ -196,9 +191,8 @@ class EntryFrame():
         self.address_label.grid(column = 0, row = 6 )
         self.address_entry.grid(column = 1 ,row = 6 )
 
-        self.note_label.grid(column = 0, row = 7 )
-        self.note_text.grid(column = 1 ,row = 7 )
+        self.cmt_label.grid(column = 0, row = 7 )
+        self.cmt_text.grid(column = 1 ,row = 7 )
         # save_button
         self.save_button.grid( column = 1, row= 8)
-
         self.frame.grid( column = 1, row = 0, sticky = N )
